@@ -86,16 +86,40 @@
 //!
 //! ## Features
 //!
-//! - `const_generics`: Extends the implementations of [`CheckBytes`] to all
-//!   arrays and not just arrays up to length 32 (enabled by default).
-//! - `full_errors`: Some validation algorithms are optimized for speed and do not report full error
+//! - `verbose`: Some validation algorithms are optimized for speed and do not report full error
 //!   details by default. This feature provides full error information.
-//! - `log`: Logs errors using the `log` crate in `no_std` environments. Does nothing in `std`
-//!   environments.
-//! - `std`: Enables standard library support (enabled by default).
+//! - `std`: Enables standard library support (enabled by default). If the `std` feature is not
+//!   enabled, the `alloc` crate is required.
+//!
+//! ## Crate support
+//!
+//! Some common crates need to be supported by bytecheck before an official integration has been
+//! made. Support is provided by bytecheck for these crates, but in the future crates should depend
+//! on bytecheck and provide their own implementations. The crates that already have support
+//! provided by bytecheck should work toward integrating the implementations into themselves.
+//!
+//! Crates supported by bytecheck:
+//!
+//! - [`uuid`](https://docs.rs/uuid)
 
+#![deny(broken_intra_doc_links)]
+#![deny(missing_docs)]
+#![deny(missing_crate_level_docs)]
 #![cfg_attr(not(feature = "std"), no_std)]
 
+#[cfg(not(feature = "std"))]
+extern crate alloc;
+
+// Support for various common crates. These are primarily to get users off the ground and build some
+// momentum.
+
+// These are NOT PLANNED to remain in bytecheck for the final release. Much like serde, these
+// implementations should be moved into their respective crates over time. Before adding support for
+// another crate, please consider getting bytecheck support in the crate instead.
+
+#[cfg(feature = "uuid")]
+pub mod uuid;
+
 #[cfg(has_atomics)]
 use core::sync::atomic::{
     AtomicBool, AtomicI16, AtomicI32, AtomicI8, AtomicU16, AtomicU32, AtomicU8,
@@ -103,7 +127,7 @@ use core::sync::atomic::{
 #[cfg(has_atomics_64)]
 use core::sync::atomic::{AtomicI64, AtomicU64};
 use core::{
-    convert::TryFrom,
+    convert::{Infallible, TryFrom},
     fmt,
     marker::{PhantomData, PhantomPinned},
     num::{
@@ -113,25 +137,53 @@ use core::{
     ops, ptr, slice,
 };
 use ptr_meta::PtrExt;
-#[cfg(not(feature = "full_errors"))]
+#[cfg(not(feature = "verbose"))]
 use simdutf8::basic::{from_utf8, Utf8Error};
-#[cfg(feature = "full_errors")]
+#[cfg(feature = "verbose")]
 use simdutf8::compat::{from_utf8, Utf8Error};
-#[cfg(feature = "std")]
-use std::error::Error;
 
 pub use bytecheck_derive::CheckBytes;
 
+/// An error that can be debugged and displayed.
+///
+/// With the `std` feature, this also supports `std::error::Error`.
+#[cfg(not(feature = "std"))]
+pub trait Error: fmt::Debug + fmt::Display + 'static {}
+
+#[cfg(not(feature = "std"))]
+impl<T: fmt::Debug + fmt::Display + 'static> Error for T {}
+
+/// An error that can be debugged and displayed.
+///
+/// With the `std` feature, this also supports `std::error::Error`.
+#[cfg(feature = "std")]
+pub trait Error: std::error::Error + 'static {
+    /// Gets this error as an `std::error::Error`.
+    fn as_error(&self) -> &(dyn std::error::Error + 'static);
+}
+
+#[cfg(feature = "std")]
+impl<T: std::error::Error + 'static> Error for T {
+    fn as_error(&self) -> &(dyn std::error::Error + 'static) {
+        self
+    }
+}
+
+/// The type used for boxing errors.
+#[cfg(not(feature = "std"))]
+pub type ErrorBox<E> = alloc::boxed::Box<E>;
+
+/// The type used for boxing errors.
+#[cfg(feature = "std")]
+pub type ErrorBox<E> = std::boxed::Box<E>;
+
 /// A type that can check whether a pointer points to a valid value.
 ///
 /// `CheckBytes` can be derived with [`CheckBytes`](macro@CheckBytes) or
 /// implemented manually for custom behavior.
 pub trait CheckBytes<C: ?Sized> {
     /// The error that may result from checking the type.
-    #[cfg(feature = "std")]
     type Error: Error + 'static;
-    #[cfg(not(feature = "std"))]
-    type Error: fmt::Debug + fmt::Display + 'static;
 
     /// Checks whether the given pointer points to a valid value within the
     /// given context.
@@ -144,90 +196,10 @@ pub trait CheckBytes<C: ?Sized> {
         -> Result<&'a Self, Self::Error>;
 }
 
-// std error handling
-
-/// The type of nested errors
-#[cfg(feature = "std")]
-pub type NestedError = Box<dyn Error>;
-
-/// Nests errors according to user configuration
-#[cfg(feature = "std")]
-#[inline]
-pub fn handle_error<E: Error + 'static>(error: E) -> NestedError {
-    Box::new(error)
-}
-
-// no_std error handling
-
-/// An error type that reminds users to enable logging
-#[derive(Debug)]
-pub struct EnableLoggingError;
-
-impl fmt::Display for EnableLoggingError {
-    #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "enable the `log` feature in bytecheck to log errors")
-    }
-}
-
-// logless error handling
-
-/// The type of nested errors
-#[cfg(all(not(feature = "std"), not(feature = "log")))]
-pub type NestedError = EnableLoggingError;
-
-/// Nests errors according to user configuration
-#[cfg(all(not(feature = "std"), not(feature = "log")))]
-#[inline]
-pub fn handle_error<E>(_: E) -> NestedError {
-    EnableLoggingError
-}
-
-/// An error type that reminds users to check the logs
-#[derive(Debug)]
-pub struct CheckLogsError;
-
-impl fmt::Display for CheckLogsError {
-    #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "see logs")
-    }
-}
-
-// logging error handling
-
-/// The type of nested errors
-#[cfg(all(not(feature = "std"), feature = "log"))]
-pub type NestedError = CheckLogsError;
-
-/// Nests errors according to user configuration
-#[cfg(all(not(feature = "std"), feature = "log"))]
-pub fn handle_error<E: Display>(error: E) -> NestedError {
-    log::error!("[bytecheck] {}", error);
-    CheckLogsError
-}
-
-/// An error that cannot be produced
-///
-/// This is used primarily for primitive types that do not have invalid values
-/// such as integers and floats.
-#[derive(Debug)]
-pub enum Unreachable {}
-
-impl fmt::Display for Unreachable {
-    #[inline]
-    fn fmt(&self, _f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        unsafe { core::hint::unreachable_unchecked() }
-    }
-}
-
-#[cfg(feature = "std")]
-impl Error for Unreachable {}
-
 macro_rules! impl_primitive {
     ($type:ty) => {
         impl<C: ?Sized> CheckBytes<C> for $type {
-            type Error = Unreachable;
+            type Error = Infallible;
 
             #[inline]
             unsafe fn check_bytes<'a>(
@@ -271,7 +243,7 @@ impl_primitive!(AtomicU32);
 impl_primitive!(AtomicU64);
 
 impl<T: ?Sized, C: ?Sized> CheckBytes<C> for PhantomData<T> {
-    type Error = Unreachable;
+    type Error = Infallible;
 
     #[inline]
     unsafe fn check_bytes<'a>(value: *const Self, _: &mut C) -> Result<&'a Self, Self::Error> {
@@ -280,7 +252,7 @@ impl<T: ?Sized, C: ?Sized> CheckBytes<C> for PhantomData<T> {
 }
 
 impl<C: ?Sized> CheckBytes<C> for PhantomPinned {
-    type Error = Unreachable;
+    type Error = Infallible;
 
     #[inline]
     unsafe fn check_bytes<'a>(value: *const Self, _: &mut C) -> Result<&'a Self, Self::Error> {
@@ -309,7 +281,7 @@ impl fmt::Display for BoolCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for BoolCheckError {}
+impl std::error::Error for BoolCheckError {}
 
 impl<C: ?Sized> CheckBytes<C> for bool {
     type Error = BoolCheckError;
@@ -352,9 +324,9 @@ pub struct CharCheckError {
     pub invalid_value: u32,
 }
 
-impl From<Unreachable> for CharCheckError {
+impl From<Infallible> for CharCheckError {
     #[inline]
-    fn from(_: Unreachable) -> Self {
+    fn from(_: Infallible) -> Self {
         unsafe { core::hint::unreachable_unchecked() }
     }
 }
@@ -370,7 +342,7 @@ impl fmt::Display for CharCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for CharCheckError {}
+impl std::error::Error for CharCheckError {}
 
 impl<C: ?Sized> CheckBytes<C> for char {
     type Error = CharCheckError;
@@ -396,7 +368,10 @@ macro_rules! impl_tuple {
         /// An error resulting from an invalid tuple.
         #[derive(Debug)]
         pub enum $error<$($type),+> {
-            $($type($type),)+
+            $(
+                /// The given tuple member was invalid.
+                $type($type),
+            )+
         }
 
         impl<$($type: fmt::Display),*> fmt::Display for $error<$($type),+> {
@@ -410,7 +385,7 @@ macro_rules! impl_tuple {
         }
 
         #[cfg(feature = "std")]
-        impl<$($type: fmt::Display + fmt::Debug),*> Error for $error<$($type),+> {}
+        impl<$($type: fmt::Display + fmt::Debug),*> std::error::Error for $error<$($type),+> {}
 
         impl<$($type: CheckBytes<C>,)+ C: ?Sized> CheckBytes<C> for ($($type,)+) {
             type Error = $error<$($type::Error),+>;
@@ -457,34 +432,8 @@ impl<T: fmt::Display> fmt::Display for ArrayCheckError<T> {
 }
 
 #[cfg(feature = "std")]
-impl<T: fmt::Debug + fmt::Display> Error for ArrayCheckError<T> {}
+impl<T: fmt::Debug + fmt::Display> std::error::Error for ArrayCheckError<T> {}
 
-#[cfg(not(feature = "const_generics"))]
-macro_rules! impl_array {
-    () => {};
-    ($len:expr, $($rest:expr,)*) => {
-        impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for [T; $len] {
-            type Error = ArrayCheckError<T::Error>;
-
-            #[inline]
-            unsafe fn check_bytes<'a>(value: *const Self, context: &mut C) -> Result<&'a Self, Self::Error> {
-                #[allow(clippy::reversed_empty_ranges)]
-                for index in 0..$len {
-                    let el = value.cast::<T>().add(index);
-                    T::check_bytes(el, context).map_err(|error| ArrayCheckError { index, error })?;
-                }
-                Ok(&*value)
-            }
-        }
-
-        impl_array! { $($rest,)* }
-    };
-}
-
-#[cfg(not(feature = "const_generics"))]
-impl_array! { 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, }
-
-#[cfg(feature = "const_generics")]
 impl<T: CheckBytes<C>, C: ?Sized, const N: usize> CheckBytes<C> for [T; N] {
     type Error = ArrayCheckError<T::Error>;
 
@@ -525,7 +474,7 @@ impl<T: fmt::Display> fmt::Display for SliceCheckError<T> {
 }
 
 #[cfg(feature = "std")]
-impl<T: fmt::Debug + fmt::Display> Error for SliceCheckError<T> {}
+impl<T: fmt::Debug + fmt::Display> std::error::Error for SliceCheckError<T> {}
 
 impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for [T] {
     type Error = SliceCheckError<T::Error>;
@@ -568,9 +517,9 @@ impl fmt::Display for StrCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for StrCheckError {
+impl std::error::Error for StrCheckError {
     #[inline]
-    fn source(&self) -> Option<&(dyn Error + 'static)> {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
             StrCheckError::Utf8Error(e) => Some(e),
         }
@@ -594,7 +543,7 @@ pub struct StructCheckError {
     /// The name of the struct field that was invalid
     pub field_name: &'static str,
     /// The error that occurred while validating the field
-    pub inner: NestedError,
+    pub inner: ErrorBox<dyn Error>,
 }
 
 impl fmt::Display for StructCheckError {
@@ -609,7 +558,7 @@ impl fmt::Display for StructCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for StructCheckError {}
+impl std::error::Error for StructCheckError {}
 
 /// An error resulting from an invalid tuple struct.
 #[derive(Debug)]
@@ -617,7 +566,7 @@ pub struct TupleStructCheckError {
     /// The index of the struct field that was invalid
     pub field_index: usize,
     /// The error that occurred while validating the field
-    pub inner: NestedError,
+    pub inner: ErrorBox<dyn Error>,
 }
 
 impl fmt::Display for TupleStructCheckError {
@@ -632,7 +581,7 @@ impl fmt::Display for TupleStructCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for TupleStructCheckError {}
+impl std::error::Error for TupleStructCheckError {}
 
 /// An error resulting from an invalid enum.
 #[derive(Debug)]
@@ -684,7 +633,7 @@ impl<T: fmt::Display> fmt::Display for EnumCheckError<T> {
 }
 
 #[cfg(feature = "std")]
-impl<T: fmt::Debug + fmt::Display> Error for EnumCheckError<T> {}
+impl<T: fmt::Debug + fmt::Display> std::error::Error for EnumCheckError<T> {}
 
 // Range types
 impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::Range<T> {
@@ -698,13 +647,13 @@ impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::Range<T> {
         T::check_bytes(ptr::addr_of!((*value).start), context).map_err(|error| {
             StructCheckError {
                 field_name: "start",
-                inner: handle_error(error),
+                inner: ErrorBox::new(error),
             }
         })?;
         T::check_bytes(ptr::addr_of!((*value).end), context).map_err(|error| {
             StructCheckError {
                 field_name: "end",
-                inner: handle_error(error),
+                inner: ErrorBox::new(error),
             }
         })?;
         Ok(&*value)
@@ -723,7 +672,7 @@ impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::RangeFrom<T> {
         T::check_bytes(ptr::addr_of!((*value).start), context).map_err(|error| {
             StructCheckError {
                 field_name: "start",
-                inner: handle_error(error),
+                inner: ErrorBox::new(error),
             }
         })?;
         Ok(&*bytes.cast())
@@ -731,7 +680,7 @@ impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::RangeFrom<T> {
 }
 
 impl<C: ?Sized> CheckBytes<C> for ops::RangeFull {
-    type Error = Unreachable;
+    type Error = Infallible;
 
     #[inline]
     unsafe fn check_bytes<'a>(value: *const Self, _: &mut C) -> Result<&'a Self, Self::Error> {
@@ -750,7 +699,7 @@ impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::RangeTo<T> {
         T::check_bytes(ptr::addr_of!((*value).end), context).map_err(|error| {
             StructCheckError {
                 field_name: "end",
-                inner: handle_error(error),
+                inner: ErrorBox::new(error),
             }
         })?;
         Ok(&*value)
@@ -768,7 +717,7 @@ impl<T: CheckBytes<C>, C: ?Sized> CheckBytes<C> for ops::RangeToInclusive<T> {
         T::check_bytes(ptr::addr_of!((*value).end), context).map_err(|error| {
             StructCheckError {
                 field_name: "end",
-                inner: handle_error(error),
+                inner: ErrorBox::new(error),
             }
         })?;
         Ok(&*value)
@@ -782,9 +731,9 @@ pub enum NonZeroCheckError {
     IsZero,
 }
 
-impl From<Unreachable> for NonZeroCheckError {
+impl From<Infallible> for NonZeroCheckError {
     #[inline]
-    fn from(_: Unreachable) -> Self {
+    fn from(_: Infallible) -> Self {
         unsafe { core::hint::unreachable_unchecked() }
     }
 }
@@ -799,7 +748,7 @@ impl fmt::Display for NonZeroCheckError {
 }
 
 #[cfg(feature = "std")]
-impl Error for NonZeroCheckError {}
+impl std::error::Error for NonZeroCheckError {}
 
 macro_rules! impl_nonzero {
     ($nonzero:ident, $underlying:ident) => {
